//+------------------------------------------------------------------+
//|                                             SidewaysDetector.mq4 |
//|                                       Copyright (c) 2017 mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
//| Need a custom Indicator or EA for MT4, MT5, cAlgo, NinjaTrader?  |
//| Contact me at mwfx108@gmail.com to ask for details and prices.   |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| SidewaysDetector_TRO_MODIFIED_VERSION                            |
//| MODIFIED BY AVERY T. HORTON, JR. AKA THERUMPLEDONE@GMAIL.COM     |
//| I am NOT the ORIGINAL author 
//  and I am not claiming authorship of this indicator. 
//  All I did was modify it. I hope you find my modifications useful.|
//|                                                                  |
//+------------------------------------------------------------------+

#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property version   "1.06"
#property strict
#property indicator_chart_window
//+------------------------------------------------------------------+
//| Variables and Types                                              |
//+------------------------------------------------------------------+
struct Zone
{
   int Start;
   int End;
   double Low;
   double High;
};
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
// Multiplier for the average zone height. The bigger, the lower the zones
extern double  Average_Weight = 1.0;
// Number of consecutive bars within a zone
extern int     Min_Zone_Width = 3;
// Tolerance in points above a low / high of a bar
extern int     Tolerance = 1;
// Color of the zone
extern color   Rectangle_Color = clrLightGray;
// Number of last zones to show. 0 = Show all zones
extern int     Show_N_Last_Zones = 50;
// Number of bars to process. 0 = All
extern int     Num_Bars = 500;

extern bool   Show_Lines  = true ;
extern bool   Show_Lines_Right = false ;
extern bool   Show_Labels = true ;
extern bool   Show_Boxes  = true ;
extern bool   Show_Level  = false ;
extern bool   Show_Side_Markers = false ;

extern int corner         = 3;
extern int price_x_offset = 10 ; 
extern int price_y_offset = 100 ; 

extern string myFont          = "Terminal" ;
extern int   myFontSize       = 20;


extern bool   AllLinesBackground = true ;   
input ENUM_LINE_STYLE    Line_Style    = STYLE_SOLID ;
extern int    Line_Width    = 1 ;
extern color  Line_Color_Hi = Lime ;
extern color  Line_Color_Mid = Aqua ;
extern color  Line_Color_Lo = Red ;
extern color  Message_Color = clrMagenta ;


string symbol ;  

int    digits, period ; 

double point, mid ;
//+------------------------------------------------------------------+
int OnInit()
{

  OnDeinit(0);

   return(INIT_SUCCEEDED);
} 
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   period       =  Period() ;
   symbol       =  Symbol() ;
   digits       =  Digits ;
   point        =  Point ;
  
   if(digits == 5 || digits == 3) { digits = digits - 1 ; point = point * 10 ; } 
  
   RemoveObjects();
   TRO();
  
//---
} 
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
  
   int numBars = Num_Bars > 0 ? Num_Bars : rates_total;
   Zone zones[];
  
   GetZones(zones, numBars);
   DrawZones(zones);
   
   
   
   ArrayFree(zones);
  
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Get sideways zones.                                              |
//| This algorithm detects sideways markets and is based on the      |
//| assumption that the markets always move sideways, except when    |
//| they are not.                                                    |
//+------------------------------------------------------------------+
void GetZones(Zone &zones[], int numBars)
{
//---
  
   double avgZoneHeight = 0, tolerancePt = Tolerance * Point;
  
   // --- Two passes:
   // Pass 1: Get average height of zone to filter big movements as the beginning of a zone
   // Pass 2: Add valid zones to a list
   for (int pass = 1; pass <= 2; pass++)
   {
      bool hasBrokenRange = false;
      double rangeHigh = High[numBars - 1] + tolerancePt, rangeLow = Low[numBars - 1] - tolerancePt;
      int zoneStart = numBars - 1, zoneEnd = 0, zoneSize = 0, zonesCounter = 0;
      
      ArrayFree(zones);
      
      // Iterate through all bars, beginning at the very first bar of the chart
      for (int i = numBars - 2; i >= 0; i--)
      {
         zoneSize = zoneStart - i;
         zoneEnd = i;
        
         // Check if a bar opened or closed above range high or range low
         if (Open[i] > rangeHigh || Open[i] < rangeLow || Close[i] > rangeHigh || Close[i] < rangeLow)
         {
            // Check if number of bars in zone is bigger or equal the required width
            if (zoneSize >= Min_Zone_Width)
            {
               // On first pass, add up the height of all zones found
               if (pass == 1)
               {
                  avgZoneHeight += MathAbs(rangeHigh - rangeLow);
               }
              
               // On second pass, add the zone to a list
               if (pass == 2)
                  AddZone(zones, zoneStart, zoneEnd + 1, rangeLow, rangeHigh);
                  
               zonesCounter++;
            }
            
            hasBrokenRange = true;
            zoneStart = i;
            rangeHigh = High[i] + tolerancePt;
            rangeLow = Low[i] - tolerancePt;
            
            // Pass 2: If bar height is bigger than the expected average zone height, skip this bar
            if (pass == 2 && MathAbs(High[i] - Low[i]) >= avgZoneHeight)
            {
               zoneStart = -1;
               rangeHigh = 0;
               rangeLow = 0;
               continue;
            }
         }
         else
            hasBrokenRange = false;
      }
      
      // After first pass, calculate the average zone height
      // and weight it a bit more to filter out big movements
      if (pass == 1)
         avgZoneHeight /= zonesCounter * Average_Weight;
      
      // Check if we have an open zone after pass 2
      if (pass == 2 && !hasBrokenRange && zoneSize >= Min_Zone_Width)
         AddZone(zones, zoneStart, 0, rangeLow, rangeHigh);
   }
  
//---
}
//+------------------------------------------------------------------+
//| Display zones on the chart                                       |
//+------------------------------------------------------------------+
void DrawZones(Zone &zones[])
{
//---  
    
    
   RemoveObjects();
   int zonesCounter = ArraySize(zones);
   for (int i = 0; i < zonesCounter; i++)
   {
      // Show the N last zones only
      if (Show_N_Last_Zones > 0 && i < zonesCounter - Show_N_Last_Zones)
         continue;
        
      string objName = "SD_Zone_" + IntegerToString(i);
      Zone zone = zones[i];
      ObjectCreate(objName, OBJ_RECTANGLE, 0, Time[zone.Start], zone.High, Time[zone.End], zone.Low);
      ObjectSetInteger(0, objName, OBJPROP_COLOR, Rectangle_Color);
      ObjectSetInteger(0, objName, OBJPROP_BACK, 1);


      if(Close[0] <= zone.High && Close[0] >= zone.Low )
      {
         setObject("SD_Zone_msg","Boxing"   ,price_x_offset,price_y_offset, Message_Color, myFont, myFontSize );      
      }
      else
      {
         setObject("SD_Zone_msg","Out of Box"   ,price_x_offset,price_y_offset, Message_Color, myFont, myFontSize );      
      }   

      drawLine(zone.High ,"SD_Zone_H", Line_Color_Hi , Line_Style,"Box High  ", Time[zone.Start]  ) ;
      drawLine(zone.Low ,"SD_Zone_L", Line_Color_Lo , Line_Style,"Box Low  ",Time[zone.Start]  ) ;    

      mid = (zone.High + zone.Low) * 0.50 ; 
      drawLine(mid ,"SD_Zone_M", Line_Color_Mid , Line_Style,"Box Mid  ",Time[zone.Start]  ) ;    

      
   }
  
//---
}
//+------------------------------------------------------------------+
//| Add a zone to the list                                           |
//+------------------------------------------------------------------+
void AddZone(Zone &zones[], int start, int end, double low, double high)
{
//---
  
   int size = ArraySize(zones);
   ArrayResize(zones, size + 1);
   zones[size].Start = start;
   zones[size].End = end;
   zones[size].Low = low;
   zones[size].High = high;
  
//---  
}  
//+------------------------------------------------------------------+
//| Remove our chart objects                                         |
//+------------------------------------------------------------------+
void RemoveObjects()
{
//---
  
   for (int i = ObjectsTotal(); i >= 0; i--)
   {
      string objName = ObjectName(i);
      if (StringFind(objName, "SD_Zone_") >= 0)
         ObjectDelete(objName);
   }
  
//---
} 
//+------------------------------------------------------------------+  

void setObject(string labelName,string text,int x,int y,color theColor, string font = "Courier New",int size=10,int angle=0)
{
 
      
      if (ObjectFind(labelName) == -1)
          {
             ObjectCreate(labelName,OBJ_LABEL,0,0,0);
             ObjectSet(labelName,OBJPROP_CORNER,corner);
             if (angle != 0)
                  ObjectSet(labelName,OBJPROP_ANGLE,angle);
          }               
       ObjectSet(labelName,OBJPROP_XDISTANCE,x);
       ObjectSet(labelName,OBJPROP_YDISTANCE,y);
       ObjectSetText(labelName,text,size,font,theColor);
}
//+------------------------------------------------------------------+
 

string fFill(string filled, int f ) 
{
   string FILLED ;
   
   FILLED = StringSubstr(filled + "                                         ",0,f) ;
   
return(FILLED);
}

//+------------------------------------------------------------------+
void drawLine(double lvl,string name, color Col,int type,string text, datetime sTime)
{
   datetime StartTime, EndTime;     
   string   pipdist ;
   bool     RayRight ;
   int      theStyle ;

   StartTime = sTime; 
 
// ADJUSTING FOR BEGINNING OF WEEK/MONTH FOR PLOTTING
   if(period > PERIOD_D1)
   {        
      int  day1bar   = iBarShift(symbol,PERIOD_D1,StartTime,true);      
   
      if(day1bar == -1) 
      {        
         StartTime = StartTime+86400 ;
         day1bar   = iBarShift(symbol,PERIOD_D1,StartTime,true);  
         
         if(day1bar == -1) 
         {        
            StartTime = StartTime+86400 ;    
         }   
      } 
   } 
 

      theStyle = Line_Style ;
   
   
   EndTime  = Time[0] ;
   RayRight = false ;
   
   if( Show_Lines_Right )
   { 
      StartTime   = Time[1] ;
      EndTime     = Time[0] ;
      RayRight    = TRUE ;   
   }  
 
   ObjectDelete(name);
   ObjectCreate(name, OBJ_TREND, 0, StartTime, lvl,EndTime,lvl);
   ObjectSet(name, OBJPROP_STYLE, theStyle);    
   ObjectSet(name, OBJPROP_COLOR, Col);        
   ObjectSet(name,OBJPROP_WIDTH,Line_Width);
   ObjectSet(name,OBJPROP_BACK,AllLinesBackground); 
   ObjectSet(name,OBJPROP_RAY_RIGHT,RayRight); 
   
     
            
   if(Show_Level)
   {
            ObjectDelete(name+"X");
            ObjectCreate(name+"X", OBJ_HLINE, 0, 0, lvl );
            ObjectSet(name+"X", OBJPROP_STYLE, 5);    
            ObjectSet(name+"X", OBJPROP_COLOR, Col);        
   }
   
     //Determine the current chart scale (chart scale number should be 0-5) 
     

   if(Show_Labels)
   {         
     string Obj0002 = name+"linelbl" ;
     double lclose  ;
     
     int Chart_Scale  ,Bar_Width  ;      
     Chart_Scale = ChartScaleGet();
     
     //Set bar widths             
           if(Chart_Scale == 0) {Bar_Width = 120;}
     else {if(Chart_Scale == 1) {Bar_Width = 60;}      
     else {if(Chart_Scale == 2) {Bar_Width = 30;}
     else {if(Chart_Scale == 3) {Bar_Width = 15;}
     else {if(Chart_Scale == 4) {Bar_Width = 8;}
     else {Bar_Width = 4;} }}}}
 
     datetime LabelTime; 
     int LabelPeriod;
     
     if(Period() < PERIOD_MN1) { LabelPeriod = Period(); } else { LabelPeriod = PERIOD_MN1 + 10000 ; } 
     
     LabelTime = Time[0]+LabelPeriod*60*Bar_Width ; // Time[WindowFirstVisibleBar()/2]; // Time[0]+Period()*60*ShiftLabel ;   


         lclose  = iClose(symbol,PERIOD_D1,0) ;
         pipdist = sDiff( lclose , lvl  )   ;   

          
      ObjectDelete(Obj0002);
      
      if(ObjectFind(Obj0002) != 0)
      {
         ObjectCreate(Obj0002, OBJ_TEXT, 0, LabelTime, lvl);
         ObjectSetText(Obj0002,fFill(text+pipdist,22) , 8, "Terminal", Col);
      }
      else
      {
         ObjectMove(Obj0002, 0, LabelTime, lvl);
      } // if 
                 
   } // if     
   
   if(Show_Boxes)
   {  
     string dName = name+"boxes" ;
     if (ObjectFind(dName) != 0)
      {
          ObjectCreate(dName,OBJ_ARROW,0,Time[0],lvl);
          ObjectSet(dName,OBJPROP_ARROWCODE,SYMBOL_RIGHTPRICE);
          ObjectSet(dName,OBJPROP_COLOR,Col);  
      } 
      else
      {
         ObjectMove(dName,0,Time[0],lvl);
         ObjectSet(dName,OBJPROP_COLOR,Col);  
      } 
   } // if   
   
   if(Show_Side_Markers)
   {      
      ObjectDelete(name+"sm") ;
      ObjectCreate(name+"sm",OBJ_HLINE,0,0,lvl);
      ObjectSet(name+"sm",OBJPROP_COLOR,Col);
      ObjectSet(name+"sm",OBJPROP_HIDDEN,FALSE);
      ObjectSet(name+"sm",OBJPROP_STYLE,5);         
      ObjectSet(name+"sm",OBJPROP_BACK,false);  
   }        
           
}
//+-------------------------------------------------------------------------------------------+
//| Subroutine:  Get the chart scale number                                                   |
//+-------------------------------------------------------------------------------------------+
int ChartScaleGet()
{
  long result = -1;
  ChartGetInteger(0,CHART_SCALE,0,result);
  return((int)result);
}

//+------------------------------------------------------------------+
string sDiff( double d1, double d2)
{   
   
   double d0 ;
   
   d0 = ( d1 - d2 ) / point ;

   return( DoubleToStr(d0,0) );
}
//+------------------------------------------------------------------+  
void TRO()
{   
   
   string TRO_OBJ    = "TROTAG"  ;  
   ObjectCreate(TRO_OBJ, OBJ_LABEL, 0, 0, 0); 
   ObjectSetText(TRO_OBJ, CharToStr(78) , 12 ,  "Wingdings",  clrGold );
   ObjectSet(TRO_OBJ, OBJPROP_CORNER, 3);
   ObjectSet(TRO_OBJ, OBJPROP_XDISTANCE, 5 );
   ObjectSet(TRO_OBJ, OBJPROP_YDISTANCE, 5 );  
}
//+------------------------------------------------------------------+
/*

   Comment(
   "i=  " , i, "\n" ,
      "Close:" +  DoubleToStr( Close[0] , Digits) , "\n" ,
   "zone.High:" +  DoubleToStr( zone.High , Digits) , "\n" ,
   "zone.Low:" +  DoubleToStr( zone.Low , Digits) , "\n" ,    

 
   
   "") ;         
          





*/
